@@ -17,10 +17,12 @@ from __future__ import absolute_import
 
 import sys
 
+from tobiko.cmd import _http_ping
 from tobiko.cmd import _ping
 from tobiko.cmd import _main
 
 main = _main.main
+TobikoHttpPing = _http_ping.TobikoHttpPing
 TobikoPing = _ping.TobikoPing
 
 if __name__ == '__main__':
